@@ -32,10 +32,7 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */
 #include <the_Foundation/intset.h>
 
 void init_ListItem(iListItem *d) {
-    d->isSeparator  = iFalse;
-    d->isSelected   = iFalse;
-    d->isDraggable  = iFalse;
-    d->isDropTarget = iFalse;
+    iZap(d->flags);
 }
 
 void deinit_ListItem(iListItem *d) {
@@ -94,6 +91,7 @@ void init_ListWidget(iListWidget *d) {
     d->itemHeight = 0;
     d->scrollMode = normal_ScrollMode;
     d->noHoverWhileScrolling = iFalse;
+    d->hideItemOnDrag = iFalse;
     init_PtrArray(&d->items);
     d->cursorItem = iInvalidPos;
     d->hoverItem = iInvalidPos;
@@ -112,6 +110,7 @@ void init_ListWidget(iListWidget *d) {
 void deinit_ListWidget(iListWidget *d) {
     removeTicker_App(refreshWhileScrolling_ListWidget_, d);
     clear_ListWidget(d);
+    deinit_IntSet(&d->invalidItems);
     deinit_PtrArray(&d->items);
     delete_VisBuf(d->visBuf);
 }
@@ -123,7 +122,7 @@ void invalidate_ListWidget(iListWidget *d) {
 }
 
 void invalidateItem_ListWidget(iListWidget *d, size_t index) {
-    insert_IntSet(&d->invalidItems, index);
+    insert_IntSet(&d->invalidItems, (int) index);
     refresh_Widget(d);
 }
 
@@ -166,7 +165,7 @@ void updateVisible_ListWidget(iListWidget *d) {
        its visibility unless it knows its correct size. */
     arrange_Widget(as_Widget(d->scroll));
     setMax_SmoothScroll(&d->scrollY, scrollMax_ListWidget_(d));
-    setRange_ScrollWidget(d->scroll, (iRangei){ 0, d->scrollY.max });    
+    setRange_ScrollWidget(d->scroll, (iRangei){ 0, d->scrollY.max });
     setThumb_ScrollWidget(d->scroll,
                           pos_SmoothScroll(&d->scrollY),
                           contentSize > 0 ? height_Rect(bounds_Widget(as_Widget(d->scroll))) *
@@ -229,13 +228,18 @@ void scrollToItem_ListWidget(iListWidget *d, size_t index, uint32_t span) {
         return;
     }
     stop_Anim(&d->scrollY.pos);
-    const iRect rect    = innerBounds_Widget(as_Widget(d));
+    const iRect rect    = innerBoundsWithoutVisualOffset_Widget(as_Widget(d));
     int         yTop    = d->itemHeight * index - pos_SmoothScroll(&d->scrollY);
     int         yBottom = yTop + d->itemHeight;
-    if (yBottom > height_Rect(rect)) {
-        scrollOffsetSpan_ListWidget(d, yBottom - height_Rect(rect), span);
+    if (height_Rect(rect) > d->itemHeight) {
+        if (yBottom > height_Rect(rect)) {
+            scrollOffsetSpan_ListWidget(d, yBottom - height_Rect(rect), span);
+        }
+        else if (yTop < 0) {
+            scrollOffsetSpan_ListWidget(d, yTop, span);
+        }
     }
-    else if (yTop < 0) {
+    else {
         scrollOffsetSpan_ListWidget(d, yTop, span);
     }
 }
@@ -302,7 +306,7 @@ size_t hoverItemIndex_ListWidget(const iListWidget *d) {
 void setHoverItem_ListWidget(iListWidget *d, size_t index) {
     if (index < size_PtrArray(&d->items)) {
         const iListItem *item = at_PtrArray(&d->items, index);
-        if (item->isSeparator) {
+        if (item->flags.isSeparator) {
             index = iInvalidPos;
         }
     }
@@ -314,7 +318,7 @@ void setHoverItem_ListWidget(iListWidget *d, size_t index) {
     }
 }
 
-static void moveCursor_ListWidget_(iListWidget *d, int dir, uint32_t animSpan) {
+static iBool moveCursor_ListWidget_(iListWidget *d, int dir, uint32_t animSpan) {
     const size_t oldCursor = d->cursorItem;
     if (isEmpty_ListWidget(d)) {
         d->cursorItem = iInvalidPos;
@@ -325,11 +329,11 @@ static void moveCursor_ListWidget_(iListWidget *d, int dir, uint32_t animSpan) {
             d->cursorItem = 0;
         }
         d->cursorItem = iClamp((int) d->cursorItem + dir, 0, maxItem);
-        while (((const iListItem *) constItem_ListWidget(d, d->cursorItem))->isSeparator &&
+        while (((const iListItem *) constItem_ListWidget(d, d->cursorItem))->flags.isSeparator &&
                ((d->cursorItem < maxItem && dir >= 0) || (d->cursorItem > 0 && dir < 0))) {
             d->cursorItem += (dir >= 0 ? 1 : -1); /* Skip separators. */
         }
-    }    
+    }
     if (oldCursor != d->cursorItem) {
         invalidateItem_ListWidget(d, oldCursor);
         invalidateItem_ListWidget(d, d->cursorItem);
@@ -337,11 +341,12 @@ static void moveCursor_ListWidget_(iListWidget *d, int dir, uint32_t animSpan) {
     if (d->cursorItem != iInvalidPos) {
         scrollToItem_ListWidget(d, d->cursorItem, prefs_App()->uiAnimations ? animSpan : 0);
     }
+    return d->cursorItem != oldCursor;
 }
 
 void setCursorItem_ListWidget(iListWidget *d, size_t index) {
     invalidateItem_ListWidget(d, d->cursorItem);
-    d->cursorItem = 0;
+    d->cursorItem = index;
     moveCursor_ListWidget_(d, 0, 0);
 }
 
@@ -386,12 +391,18 @@ static size_t resolveDragDestination_ListWidget_(const iListWidget *d, iInt2 dst
     size_t           index = itemIndex_ListWidget(d, dstPos);
     const iListItem *item  = constItem_ListWidget(d, index);
     if (!item) {
-        index = (dstPos.y < mid_Rect(bounds_Widget(constAs_Widget(d))).y ? 0 : (numItems_ListWidget(d) - 1));
+        const iWidget *w            = constAs_Widget(d);
+        const int      widgetHeight = height_Rect(bounds_Widget(w));
+        const int      visHeight    = iMin(numItems_ListWidget(d) * d->itemHeight, widgetHeight);
+        /* Clamp to first/last item. */
+        index =
+            (dstPos.y < top_Rect(bounds_Widget(w)) + visHeight / 2 ? 0
+                                                                   : (numItems_ListWidget(d) - 1));
         item = constItem_ListWidget(d, index);
     }
     const iRect   rect = itemRect_ListWidget(d, index);
     const iRangei span = ySpan_Rect(rect);
-    if (item->isDropTarget) {
+    if (item->flags.isDropTarget) {
         const int pad = size_Range(&span) / 4;
         if (dstPos.y >= span.start + pad && dstPos.y < span.end - pad) {
             *dstKind = on_DragDestination;
@@ -417,11 +428,14 @@ static iBool endDrag_ListWidget_(iListWidget *d, iInt2 endPos) {
     enum iDragDestination dstKind;
     const size_t index = resolveDragDestination_ListWidget_(d, endPos, &dstKind);
     if (index != d->dragItem) {
+        if (d->hideItemOnDrag) {
+            ((iListItem *) item_ListWidget(d, d->dragItem))->flags.isHidden = iTrue;
+        }
         if (dstKind == on_DragDestination) {
             postCommand_Widget(d, "list.dragged arg:%zu onto:%zu", d->dragItem, index);
         }
         else {
-            postCommand_Widget(d, "list.dragged arg:%zu %s:%zu", d->dragItem, 
+            postCommand_Widget(d, "list.dragged arg:%zu %s:%zu", d->dragItem,
                                dstKind == after_DragDestination ? "after" : "before",
                                dstKind == after_DragDestination ? index - 1 : index);
         }
@@ -431,6 +445,16 @@ static iBool endDrag_ListWidget_(iListWidget *d, iInt2 endPos) {
     return iTrue;
 }
 
+static void abortDrag_ListWidget_(iListWidget *d) {
+    if (d->dragItem != iInvalidPos) {
+        stop_Anim(&d->scrollY.pos);
+        invalidateItem_ListWidget(d, d->dragItem);
+        d->dragItem = iInvalidPos;
+        redrawHoverItem_ListWidget_(d);
+        setFlags_Widget(as_Widget(d), touchDrag_WidgetFlag, iFalse); /* mobile drag handles */
+    }
+}
+
 static iBool isScrollDisabled_ListWidget_(const iListWidget *d, const SDL_Event *ev) {
     int dir = 0;
     if (ev->type == SDL_MOUSEWHEEL) {
@@ -520,25 +544,37 @@ static iBool processEvent_ListWidget_(iListWidget *d, const SDL_Event *ev) {
                 case SDLK_END: {
                     if (d->scrollMode == normal_ScrollMode) {
                         const int step = cursorKeyStep_ListWidget_(d, key);
-                        moveCursor_ListWidget_(d, step, iAbs(step) == 1 ? 0 : 150);
+                        const iBool wasChanged = moveCursor_ListWidget_(d, step, iAbs(step) == 1 ? 0 : 150);
+                        if (!wasChanged && (key == SDLK_UP || key == SDLK_DOWN)) {
+                            moveFocusWithArrows_App(ev);
+                        }
                         return iTrue;
                     }
                     return iFalse;
                 }
+                case SDLK_LEFT:
+                case SDLK_RIGHT:
+                    return moveFocusWithArrows_App(ev);
                 case SDLK_RETURN:
                 case SDLK_KP_ENTER:
                 case SDLK_SPACE:
                     if (d->cursorItem != iInvalidPos) {
                         postCommand_Widget(w,
-                                           "list.clicked arg:%zu item:%p button:%d",
+                                           "list.clicked arg:%zu item:%p button:%d keyboard:1",
                                            d->cursorItem,
                                            constCursorItem_ListWidget(d),
                                            SDL_BUTTON_LEFT);
                     }
                     return iTrue;
+                case SDLK_BACKSPACE:
+                    if (d->cursorItem != iInvalidPos) {
+                        postCommand_Widget(w, "list.delete arg:%zu item:%p",
+                                           d->cursorItem, constCursorItem_ListWidget(d));
+                    }
+                    return iTrue;
             }
         }
-    }    
+    }
     if (ev->type == SDL_MOUSEMOTION) {
         const iInt2 mousePos = init_I2(ev->motion.x, ev->motion.y);
         if (ev->motion.state == 0 /* not dragging */) {
@@ -571,7 +607,7 @@ static iBool processEvent_ListWidget_(iListWidget *d, const SDL_Event *ev) {
             }
         }
     }
-    if (ev->type == SDL_MOUSEWHEEL && isHover_Widget(w)) {
+    if (ev->type == SDL_MOUSEWHEEL && isHover_Widget(w) && ev->wheel.x == 0) {
         if (d->dragHandleWidth) {
             if (d->dragItem == iInvalidPos) {
                 const iInt2 wpos = coord_MouseWheelEvent(&ev->wheel);
@@ -583,10 +619,10 @@ static iBool processEvent_ListWidget_(iListWidget *d, const SDL_Event *ev) {
                 }
             }
         }
+        extern iWidgetClass Class_SidebarWidget;
         if (isScrollDisabled_ListWidget_(d, ev)) {
             if (ev->wheel.which == SDL_TOUCH_MOUSEID) {
                 /* TODO: Could generalize this selection of the scrollable parent. */
-                extern iWidgetClass Class_SidebarWidget;
                 iWidget *sidebar = findParentClass_Widget(w, &Class_SidebarWidget);
                 if (sidebar) {
                     transferAffinity_Touch(w, sidebar);
@@ -599,6 +635,10 @@ static iBool processEvent_ListWidget_(iListWidget *d, const SDL_Event *ev) {
         if (isPerPixel_MouseWheelEvent(&ev->wheel)) {
             stop_Anim(&d->scrollY.pos);
             moveSpan_SmoothScroll(&d->scrollY, amount, 0);
+            if (isMobile_Platform() && !hasAffinity_Touch(w) &&
+                ev->wheel.which == SDL_TOUCH_MOUSEID && !isScrollDisabled_ListWidget_(d, ev)) {
+                transferAffinity_Touch(NULL, w);
+            }
         }
         else {
             /* Traditional mouse wheel. */
@@ -608,6 +648,12 @@ static iBool processEvent_ListWidget_(iListWidget *d, const SDL_Event *ev) {
         }
         return iTrue;
     }
+    if (ev->type == SDL_MOUSEWHEEL && isHover_Widget(w) && ev->wheel.y == 0 &&
+        isPerPixel_MouseWheelEvent(&ev->wheel) && !isInertia_MouseWheelEvent(&ev->wheel)) {
+        iInt2 coord = mouseCoord_SDLEvent(ev);
+        postCommand_Widget(w, "listswipe.moved arg:%d coord:%d %d", ev->wheel.x, coord.x, coord.y);
+        return iTrue;
+    }
     switch (processEvent_Click(&d->click, ev)) {
         case started_ClickResult:
             d->noHoverWhileScrolling = iFalse;
@@ -615,12 +661,7 @@ static iBool processEvent_ListWidget_(iListWidget *d, const SDL_Event *ev) {
             redrawHoverItem_ListWidget_(d);
             return iTrue;
         case aborted_ClickResult:
-            if (d->dragItem != iInvalidPos) {
-                stop_Anim(&d->scrollY.pos);
-                invalidateItem_ListWidget(d, d->dragItem);
-                d->dragItem = iInvalidPos;
-            }
-            redrawHoverItem_ListWidget_(d);
+            abortDrag_ListWidget_(d);
             break;
         case drag_ClickResult:
             if (d->click.clickButton != SDL_BUTTON_LEFT) {
@@ -629,7 +670,7 @@ static iBool processEvent_ListWidget_(iListWidget *d, const SDL_Event *ev) {
             if (d->dragItem == iInvalidPos && length_I2(delta_Click(&d->click)) > gap_UI) {
                 const size_t over = itemIndex_ListWidget(d, d->click.startPos);
                 if (over != iInvalidPos &&
-                    ((const iListItem *) item_ListWidget(d, over))->isDraggable) {
+                    ((const iListItem *) item_ListWidget(d, over))->flags.isDraggable) {
                     d->dragItem = over;
                     d->dragOrigin = sub_I2(topLeft_Rect(itemRect_ListWidget(d, over)),
                                            d->click.startPos);
@@ -646,10 +687,11 @@ static iBool processEvent_ListWidget_(iListWidget *d, const SDL_Event *ev) {
                                             zero_I2(), init_I2(-d->dragHandleWidth, 0)),
                               pos_Click(&d->click)) &&
                 d->hoverItem != iInvalidPos) {
-                postCommand_Widget(w, "list.clicked arg:%zu button:%d item:%p",
+                postCommand_Widget(w, "list.clicked arg:%zu button:%d item:%p device:%u",
                                    d->hoverItem,
                                    d->click.clickButton,
-                                   constHoverItem_ListWidget(d));
+                                   constHoverItem_ListWidget(d),
+                                   ev->button.which);
             }
             return iTrue;
         default:
@@ -698,12 +740,6 @@ static void draw_ListWidget_(const iListWidget *d) {
             iVisBufTexture *buf = &d->visBuf->buffers[i];
             iRanges drawItems = { iMax(0, buf->origin) / d->itemHeight,
                                   iMax(0, buf->origin + d->visBuf->texSize.y) / d->itemHeight };
-#if 0
-            if (isEmpty_Rangei(buf->validRange)) {
-                beginTarget_Paint(&p, buf->texture);
-                fillRect_Paint(&p, (iRect){ zero_I2(), d->visBuf->texSize }, bg[i]);
-            }
-#endif
 #if defined (iPlatformApple)
             const int blankWidth = 0; /* scrollbars fade away */
 #else
@@ -715,8 +751,9 @@ static void draw_ListWidget_(const iListWidget *d) {
                 const size_t index = *v.value;
                 if (contains_Range(&drawItems, index) && index < size_PtrArray(&d->items)) {
                     const iListItem *item = constAt_PtrArray(&d->items, index);
-                    const iRect      itemRect = { init_I2(0, index * d->itemHeight - buf->origin),
-                                                  init_I2(d->visBuf->texSize.x, d->itemHeight) };
+                    if (item->flags.isHidden) continue;
+                    const iRect itemRect = { init_I2(0, index * d->itemHeight - buf->origin),
+                                             init_I2(d->visBuf->texSize.x, d->itemHeight) };
                     beginTarget_Paint(&p, buf->texture);
                     fillRect_Paint(&p, itemRect, bg[i]);
                     if (index != d->dragItem) {
@@ -730,10 +767,12 @@ static void draw_ListWidget_(const iListWidget *d) {
                 beginTarget_Paint(&p, buf->texture);
                 drawItems.start = invalidRange[i].start / d->itemHeight;
                 drawItems.end   = invalidRange[i].end   / d->itemHeight + 1;
-                for (size_t j = drawItems.start; j < drawItems.end && j < size_PtrArray(&d->items); j++) {
-                    const iListItem *item     = constAt_PtrArray(&d->items, j);
-                    const iRect      itemRect = { init_I2(0, j * d->itemHeight - buf->origin),
-                                                  init_I2(d->visBuf->texSize.x, d->itemHeight) };
+                for (size_t j = drawItems.start; j < drawItems.end && j < size_PtrArray(&d->items);
+                     j++) {
+                    const iListItem *item = constAt_PtrArray(&d->items, j);
+                    if (item->flags.isHidden) continue;
+                    const iRect itemRect = { init_I2(0, j * d->itemHeight - buf->origin),
+                                             init_I2(d->visBuf->texSize.x, d->itemHeight) };
                     fillRect_Paint(&p, itemRect, bg[i]);
                     if (j != d->dragItem) {
                         class_ListItem(item)->draw(item, &p, itemRect, d);
@@ -750,6 +789,7 @@ static void draw_ListWidget_(const iListWidget *d) {
     draw_VisBuf(d->visBuf, addY_I2(topLeft_Rect(bounds), -scrollY), ySpan_Rect(bounds));
     const iBool isMobile = (deviceType_App() != desktop_AppDeviceType);
     const iInt2 mousePos = mouseCoord_Window(get_Window(), isMobile ? SDL_TOUCH_MOUSEID : 0);
+    /* The dragged item is drawn independently of the rest. */
     if (d->dragItem != iInvalidPos && (isMobile || contains_Rect(bounds, mousePos))) {
         iInt2 pos = add_I2(mousePos, d->dragOrigin);
         const iListItem *item = constAt_PtrArray(&d->items, d->dragItem);
@@ -768,7 +808,7 @@ static void draw_ListWidget_(const iListWidget *d) {
                 fillRect_Paint(&p, (iRect){ addY_I2(dstRect.pos, -gap_UI / 4),
                                             init_I2(width_Rect(dstRect), gap_UI / 2) },
                                uiTextAction_ColorId);
-            }                        
+            }
         }
         p.alpha = 0x80;
         setOpacity_Text(0.5f);
@@ -778,6 +818,13 @@ static void draw_ListWidget_(const iListWidget *d) {
     }
     unsetClip_Paint(&p);
     drawBorders_Widget(w); /* background overdraws the normal borders */
+    if (isEmpty_ListWidget(d) && isFocused_Widget(d)) {
+        drawRectThickness_Paint(
+            &p,
+            adjusted_Rect(bounds, zero_I2(), init_I2(isTerminal_Platform() ? -2 : -gap_UI, 0)),
+            gap_UI / 4,
+            uiTextAction_ColorId);
+    }
     drawChildren_Widget(w);
 }
 
